Object subclass: #Voice
	instanceVariableNames: 'name sound '
	classVariableNames: 'Voices '
	poolDictionaries: ''
	category: 'Speech-Events'!
!Voice commentStamp: '<historical>' prior: 0!
I am an abstract class for speaking voices that know how to play VoiceEvents.!


!Voice methodsFor: 'initialization' stamp: 'len 9/26/1999 17:20'!
initialize
	name _ 'anonymous'! !


!Voice methodsFor: 'accessing' stamp: 'len 6/21/1998 20:07'!
name
	^ name! !

!Voice methodsFor: 'accessing' stamp: 'len 6/21/1998 20:07'!
name: aString
	name _ aString! !

!Voice methodsFor: 'accessing' stamp: 'len 9/15/1999 23:54'!
sound
	^ sound! !


!Voice methodsFor: 'converting' stamp: 'len 9/7/1999 01:26'!
+ aVoice
	"Answer the composition of the receiver with the argument."
	^ CompositeVoice new add: self; add: aVoice; yourself! !


!Voice methodsFor: 'playing' stamp: 'len 12/22/1999 03:51'!
flush
	"Play all the events in the queue."
	sound notNil ifTrue: [sound done: true; play. sound _ nil]! !

!Voice methodsFor: 'playing' stamp: 'len 12/22/1999 03:29'!
playBuffer: buffer at: time
	| tail |
	tail _ SampledSound samples: buffer samplingRate: self samplingRate.
	sound isNil
		ifTrue: [sound _ QueueSound new startTime: time - SoundPlayer bufferMSecs.
				sound add: tail; play]
		ifFalse: [sound add: tail]! !

!Voice methodsFor: 'playing' stamp: 'len 8/28/1999 23:04'!
playGesturalEvent: event at: time
	^ self! !

!Voice methodsFor: 'playing' stamp: 'len 8/28/1999 23:05'!
playPhoneticEvent: event at: time
	^ self! !

!Voice methodsFor: 'playing' stamp: 'len 9/14/1999 00:19'!
playSilenceMSecs: msecs
	Transcript cr; show: 'silence ', msecs printString, 'msecs'.
	sound isNil ifTrue: [^ self].
	sound add: (RestSound dur: msecs / 1000.0)! !

!Voice methodsFor: 'playing' stamp: 'len 12/22/1999 03:51'!
reset
	"Reset the state of the receiver."
	sound notNil ifTrue: [sound done: true. sound _ nil]! !


!Voice methodsFor: 'printing' stamp: 'len 8/28/1999 03:04'!
printOn: aStream
	aStream nextPutAll: self class name; nextPutAll: ' ('; nextPutAll: self name; nextPut: $)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Voice class
	instanceVariableNames: ''!

!Voice class methodsFor: 'instance creation' stamp: 'len 9/13/1999 00:58'!
default
	self voices isEmpty ifTrue: [^ KlattVoice new].
	^ self voices detect: [ :one | one name = 'Kurt']
		ifNone: [self voices detect: [ :one | one samplingRate >= 16000]
					ifNone: [self voices anyOne]]! !

!Voice class methodsFor: 'instance creation' stamp: 'len 6/21/1998 19:52'!
new
	^ super new initialize! !


!Voice class methodsFor: 'class initialization' stamp: 'len 6/7/1999 01:33'!
initialize
	"
	Voice initialize
	"

	Voices _ Set new! !


!Voice class methodsFor: 'accessing' stamp: 'len 6/7/1999 01:33'!
addVoice: aVoice
	^ self voices add: aVoice! !

!Voice class methodsFor: 'accessing' stamp: 'len 6/21/1999 00:51'!
doesNotUnderstand: aMessage
	self voices do: [ :each | each name asLowercase = aMessage selector asString ifTrue: [^ each]].
	^ super doesNotUnderstand: aMessage! !

!Voice class methodsFor: 'accessing' stamp: 'len 9/14/1999 00:16'!
named: aString
	^ self voices detect: [ :one | one name = aString]! !

!Voice class methodsFor: 'accessing' stamp: 'len 6/7/1999 01:33'!
voices
	^ Voices! !


Voice subclass: #CompositeVoice
	instanceVariableNames: 'voices '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!

!CompositeVoice methodsFor: 'initialization' stamp: 'len 8/28/1999 04:00'!
initialize
	super initialize.
	self voices: OrderedCollection new! !


!CompositeVoice methodsFor: 'accessing' stamp: 'len 9/13/1999 00:00'!
add: aVoice
	^ self voices add: aVoice! !

!CompositeVoice methodsFor: 'accessing' stamp: 'len 8/28/1999 04:01'!
voices
	^ voices! !

!CompositeVoice methodsFor: 'accessing' stamp: 'len 8/28/1999 04:00'!
voices: aCollection
	voices _ aCollection! !


!CompositeVoice methodsFor: 'enumerating' stamp: 'len 8/29/1999 02:21'!
do: aBlock
	self voices do: aBlock! !


!CompositeVoice methodsFor: 'playing' stamp: 'len 12/22/1999 03:51'!
flush
	"Play all the events in the queue."
	super flush.
	self do: [ :each | each flush]! !

!CompositeVoice methodsFor: 'playing' stamp: 'len 9/26/1999 17:21'!
playGesturalEvent: event at: time
	self do: [ :each | each playGesturalEvent: event at: time]! !

!CompositeVoice methodsFor: 'playing' stamp: 'len 9/26/1999 17:21'!
playPhoneticEvent: event at: time
	self do: [ :each | each playPhoneticEvent: event at: time]! !

!CompositeVoice methodsFor: 'playing' stamp: 'len 9/26/1999 17:22'!
reset
	"Reset the state of the receiver."
	super reset.
	self do: [ :each | each reset]! !


Object subclass: #VoiceEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!
!VoiceEvent commentStamp: '<historical>' prior: 0!
I am an abstract class for all events to be played on speaking Voices, such as PhoneticEvents or GesturalEvents.!


!VoiceEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 03:06'!
duration
	"Answer the duration (in seconds) of the receiver."
	^ 0! !

!VoiceEvent methodsFor: 'accessing' stamp: 'len 8/29/1999 02:18'!
voice
	"Answer the default voice for the receiver."
	^ Voice default! !


!VoiceEvent methodsFor: 'playing' stamp: 'len 8/28/1999 23:11'!
play
	^ self playOn: self voice! !

!VoiceEvent methodsFor: 'playing' stamp: 'len 8/28/1999 23:11'!
playAt: time
	^ self playOn: self voice at: time! !

!VoiceEvent methodsFor: 'playing' stamp: 'len 12/24/1999 02:23'!
playDelayed: delay
	self playAt: Time millisecondClockValue + delay! !

!VoiceEvent methodsFor: 'playing' stamp: 'len 8/28/1999 23:11'!
playOn: aVoice
	self playOn: aVoice at: Time millisecondClockValue! !

!VoiceEvent methodsFor: 'playing' stamp: 'len 8/28/1999 03:54'!
playOn: aVoice at: time
	self subclassResponsibility! !

!VoiceEvent methodsFor: 'playing' stamp: 'len 9/13/1999 00:46'!
playOn: aVoice delayed: delay
	self playOn: aVoice at: Time millisecondClockValue + delay! !


!VoiceEvent methodsFor: 'testing' stamp: 'len 8/29/1999 21:17'!
isGestural
	^ false! !

!VoiceEvent methodsFor: 'testing' stamp: 'len 8/29/1999 21:17'!
isPhonetic
	^ false! !


!VoiceEvent methodsFor: 'transforming' stamp: 'len 8/29/1999 03:13'!
compress: aNumber
	self stretch: 1.0 / aNumber! !

!VoiceEvent methodsFor: 'transforming' stamp: 'len 8/29/1999 03:13'!
stretch: aNumber
	^ self! !


VoiceEvent subclass: #CompositeEvent
	instanceVariableNames: 'timedEvents '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!

!CompositeEvent methodsFor: 'initialization' stamp: 'len 8/28/1999 23:15'!
initialize: anInteger
	timedEvents _ SortedCollection new: anInteger! !


!CompositeEvent methodsFor: 'accessing-private' stamp: 'len 8/28/1999 22:54'!
timedEvents
	^ timedEvents! !

!CompositeEvent methodsFor: 'accessing-private' stamp: 'len 8/28/1999 22:54'!
timedEvents: aCollection
	timedEvents _ aCollection! !


!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 22:57'!
add: aVoiceEvent
	^ self add: aVoiceEvent at: self lastTime! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 22:57'!
add: aVoiceEvent at: time
	^ self timedEvents add: time -> aVoiceEvent! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 22:56'!
add: aVoiceEvent delayed: time
	^ self add: aVoiceEvent at: self lastTime + time! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/29/1999 02:16'!
addAll: aCollection
	aCollection do: [ :each | self add: each].
	^ aCollection! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/29/1999 03:08'!
at: anInteger
	^ (self timedEvents at: anInteger) value! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 23:18'!
duration
	"Answer the duration (in seconds) of the receiver."
	^ self lastTime / 1000.0! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/29/1999 03:09'!
first
	^ self at: 1! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/29/1999 03:09'!
last
	^ self at: self size! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 23:17'!
lastTime
	| last |
	self isEmpty ifTrue: [^ 0].
	last _ self timedEvents last.
	^ last key + (last value duration * 1000) rounded! !

!CompositeEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 23:16'!
size
	^ self timedEvents size! !


!CompositeEvent methodsFor: 'converting' stamp: 'len 9/29/1999 02:52'!
asArray	
	^ (1 to: self size) collect: [ :each | self at: each]! !

!CompositeEvent methodsFor: 'converting' stamp: 'len 9/27/1999 22:48'!
asPHOString
	| stream |
	stream _ WriteStream on: String new.
	self do: [ :each | stream nextPutAll: each asPHOString; nextPut: Character cr].
	^ stream contents! !


!CompositeEvent methodsFor: 'copying' stamp: 'len 8/28/1999 23:16'!
copy
	| answer |
	answer _ self class new: self size.
	self timedEvents do: [ :each | answer add: each value copy at: each key].
	^ answer! !


!CompositeEvent methodsFor: 'enumerating' stamp: 'len 12/14/1999 05:49'!
detect: aBlock
	self detect: aBlock ifNone: [self error: 'event not found']! !

!CompositeEvent methodsFor: 'enumerating' stamp: 'len 12/14/1999 05:49'!
detect: aBlock ifNone: exceptionBlock
	self do: [ :each | (aBlock value: each) ifTrue: [^ each]].
	^ exceptionBlock value! !

!CompositeEvent methodsFor: 'enumerating' stamp: 'len 8/28/1999 23:09'!
do: aBlock
	self timedEvents do: [ :each | aBlock value: each value]! !


!CompositeEvent methodsFor: 'playing' stamp: 'len 12/22/1999 03:32'!
playOn: aVoice at: time
	self timedEvents do: [ :each | each value playOn: aVoice at: each key + time].
	aVoice flush! !


!CompositeEvent methodsFor: 'testing' stamp: 'len 8/28/1999 22:56'!
isEmpty
	^ self timedEvents isEmpty! !


!CompositeEvent methodsFor: 'transforming' stamp: 'len 8/29/1999 03:12'!
compress: aNumber
	self stretch: aNumber reciprocal! !

!CompositeEvent methodsFor: 'transforming' stamp: 'len 8/29/1999 03:11'!
delay: time
	self timedEvents do: [ :each | each key: each key + time]! !

!CompositeEvent methodsFor: 'transforming' stamp: 'len 9/29/1999 05:16'!
pitchBy: aNumber
	self do: [ :each | each pitchBy: aNumber]! !

!CompositeEvent methodsFor: 'transforming' stamp: 'len 8/29/1999 03:13'!
stretch: aNumber
	self do: [ :each | each stretch: aNumber].
	self timedEvents do: [ :each | each key: (each key * aNumber) rounded]! !


!CompositeEvent methodsFor: 'editing' stamp: 'len 9/28/1999 01:08'!
inspect
	"Open a OrderedCollectionInspector on the receiver.  Use basicInspect to get a normal (less useful) type of inspector."

	OrderedCollectionInspector openOn: self withEvalPane: true! !

!CompositeEvent methodsFor: 'editing' stamp: 'len 9/28/1999 01:08'!
inspectWithLabel: aLabel
	"Open a OrderedCollectionInspector on the receiver. Use basicInspect to get a normal (less useful) type of inspector."

	OrderedCollectionInspector openOn: self withEvalPane: true withLabel: aLabel! !


!CompositeEvent methodsFor: 'private' stamp: 'len 12/13/1999 02:47'!
recomputeTimes
	| oldTimedEvents |
	oldTimedEvents _ timedEvents.
	timedEvents _ SortedCollection new: oldTimedEvents size.
	oldTimedEvents do: [ :each | self add: each value]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CompositeEvent class
	instanceVariableNames: ''!

!CompositeEvent class methodsFor: 'instance creation' stamp: 'len 8/28/1999 23:14'!
new
	^ self new: 10! !

!CompositeEvent class methodsFor: 'instance creation' stamp: 'len 8/28/1999 23:15'!
new: anInteger
	^ self basicNew initialize: anInteger! !


VoiceEvent subclass: #GesturalEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!

!GesturalEvent methodsFor: 'accessing' stamp: 'len 8/29/1999 02:18'!
voice
	"Answer the default voice for the reciever."
	^ Voice voices detect: [ :one | one class == GesturalVoice] ifNone: [super voice]! !


!GesturalEvent methodsFor: 'playing' stamp: 'len 9/6/1999 00:40'!
actOn: aHeadMorph
	self subclassResponsibility! !

!GesturalEvent methodsFor: 'playing' stamp: 'len 8/28/1999 03:53'!
playOn: aVoice at: time
	aVoice playGesturalEvent: self at: time! !


!GesturalEvent methodsFor: 'testing' stamp: 'len 8/29/1999 21:18'!
isGestural
	^ true! !


GesturalEvent subclass: #GazeGesturalEvent
	instanceVariableNames: 'point '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!

!GazeGesturalEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 03:17'!
point
	^ point! !

!GazeGesturalEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 03:17'!
point: aPoint
	point _ aPoint! !


!GazeGesturalEvent methodsFor: 'playing' stamp: 'len 9/6/1999 00:42'!
actOn: aHeadMorph
	aHeadMorph face lookAt: self point! !


!GazeGesturalEvent methodsFor: 'printing' stamp: 'len 9/6/1999 00:42'!
printOn: aStream
	aStream nextPutAll: 'look at '; print: self point! !


GesturalEvent subclass: #MoodGesturalEvent
	instanceVariableNames: 'state '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!

!MoodGesturalEvent methodsFor: 'accessing' stamp: 'len 9/7/1999 02:22'!
state
	^ state! !

!MoodGesturalEvent methodsFor: 'accessing' stamp: 'len 9/12/1999 05:03'!
state: aSymbol
	state _ aSymbol asSymbol! !


!MoodGesturalEvent methodsFor: 'playing' stamp: 'len 9/7/1999 02:23'!
actOn: aHeadMorph
	aHeadMorph face perform: self state! !


!MoodGesturalEvent methodsFor: 'printing' stamp: 'len 9/7/1999 02:26'!
printOn: aStream
	aStream nextPutAll: 'set ', self state, ' mood'! !


VoiceEvent subclass: #PhoneticEvent
	instanceVariableNames: 'phoneme pitchPoints duration loudness '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!
!PhoneticEvent commentStamp: '<historical>' prior: 0!
My instances are events for a Voice.!


!PhoneticEvent methodsFor: 'accessing-private' stamp: 'len 9/27/1999 23:06'!
pitchPoints
	^ pitchPoints! !

!PhoneticEvent methodsFor: 'accessing-private' stamp: 'len 9/27/1999 23:06'!
pitchPoints: p
	pitchPoints _ p isNumber
		ifTrue: [((0.0 to: duration by: 0.035) collect: [ :time | time @ p])]
		ifFalse: [p first isPoint ifTrue: [p] ifFalse: [(p collect: [ :each | each first @ each last])]]! !


!PhoneticEvent methodsFor: 'accessing' stamp: 'len 9/27/1999 23:06'!
averagePitch
	| sum previous |
	self pitchPoints size = 1 ifTrue: [^ self pitchPoints first y].
	sum _ 0.0.
	self pitchPoints do: [ :each |
		previous isNil ifFalse: [sum _ (each y + previous y) / 2.0 * (each x - previous x) + sum].
		previous _ each].
	sum _ previous y * (self duration - previous x) + sum.
	^ sum / self duration! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 8/28/1999 23:18'!
duration
	"Answer the duration (in seconds) of the receiver."
	^ duration! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 11/30/1999 04:26'!
duration: aNumber
	"Set the duration of the receiver (in seconds)."
	((pitchPoints isNil or: [duration isNil]) or: [duration = 0])
		ifFalse: [pitchPoints _ pitchPoints collect: [ :each | each x / duration * aNumber @ each y]].
	duration _ aNumber! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 5/28/1999 01:36'!
loudness
	^ loudness! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 5/28/1999 01:37'!
loudness: aNumber
	loudness _ aNumber! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 5/28/1999 01:36'!
phoneme
	^ phoneme! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 5/28/1999 01:35'!
phoneme: aPhoneme
	phoneme _ aPhoneme! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 9/27/1999 23:03'!
pitch: aNumber
	self pitchPoints: aNumber! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 9/27/1999 23:04'!
pitchAt: time
	"Answer the pitch of the receiver at a given time. (Do linear interpolation.)"
	| xVal count x1 x2 y1 y2 |
	pitchPoints isNil ifTrue: [^ nil].
	xVal _ pitchPoints first x.
	count _ 1.
	[xVal < time]
		whileTrue: [count _ count + 1.
					count > pitchPoints size ifTrue: [^ pitchPoints last y].
					xVal _ (pitchPoints at: count) x].
	xVal = time ifTrue: [^ (pitchPoints at: count) y].
	count = 1 ifTrue: [^ pitchPoints first y].
	x1 _ (pitchPoints at: count - 1) x.
	x2 _ (pitchPoints at: count) x.
	y1 _ (pitchPoints at: count - 1) y.
	y2 _ (pitchPoints at: count) y.
	^ (time - x1) / (x2 - x1) * (y2 - y1) + y1! !

!PhoneticEvent methodsFor: 'accessing' stamp: 'len 9/27/1999 23:07'!
pitchAt: time put: aNumber
	"Set the pitch of the receiver at a given time."
	pitchPoints isNil ifTrue: [pitchPoints _ Array with: time @ aNumber. ^ self].
	pitchPoints _ pitchPoints copyWith: time @ aNumber! !


!PhoneticEvent methodsFor: 'playing' stamp: 'len 8/28/1999 03:54'!
playOn: aVoice at: time
	aVoice playPhoneticEvent: self at: time! !


!PhoneticEvent methodsFor: 'converting' stamp: 'len 12/24/1999 02:45'!
asPHOString
	| stream |
	stream _ WriteStream on: String new.
	stream
		nextPutAll: (PhonemeSet arpabetToSampa at: self phoneme); space;
		print: (self duration * 1000) rounded.
	self pitchPoints do: [ :each | stream space; print: (each x * 1000) rounded; space; print: each y rounded].
	^ stream contents! !


!PhoneticEvent methodsFor: 'copying' stamp: 'len 9/27/1999 23:01'!
copy
	^ super copy pitchPoints: self pitchPoints copy! !


!PhoneticEvent methodsFor: 'testing' stamp: 'len 9/27/1999 23:04'!
hasPitch
	"Answer true if there is a pitch contour specified for the receiver."
	^ pitchPoints notNil! !

!PhoneticEvent methodsFor: 'testing' stamp: 'len 8/29/1999 21:17'!
isPhonetic
	^ true! !


!PhoneticEvent methodsFor: 'transforming' stamp: 'len 9/27/1999 23:05'!
pitchApply: aBlock
	"Apply aBlock to the pitch points in the receiver."
	self hasPitch ifFalse: [^ self].
	pitchPoints _ pitchPoints collect: aBlock! !

!PhoneticEvent methodsFor: 'transforming' stamp: 'len 9/27/1999 23:06'!
pitchBy: aNumber
	"Multiply the receiver's pitch contour by aNumber."
	self hasPitch ifFalse: [^ self].
	pitchPoints _ pitchPoints collect: [ :each | each x @ (each y * aNumber)]! !

!PhoneticEvent methodsFor: 'transforming' stamp: 'len 6/5/1999 03:26'!
stretch: aNumber
	self duration: self duration * aNumber! !

!PhoneticEvent methodsFor: 'transforming' stamp: 'len 9/27/1999 23:05'!
transposeBy: aNumber
	"Add the given step to the receiver's pitch."
	pitchPoints _ pitchPoints collect: [ :each | each x @ (each y + aNumber)]! !


!PhoneticEvent methodsFor: 'printing' stamp: 'len 9/27/1999 23:06'!
printOn: aStream
	| first |
	aStream nextPutAll: '#('; print: phoneme; space; print: loudness; space; print: duration.
	self pitchPoints isNil ifTrue: [aStream nextPut: $). ^ self].
	aStream nextPutAll: ' #('.
	first _ true.
	self pitchPoints do: [ :each |
		first ifFalse: [aStream space].
		aStream print: each x; space; print: each y.
		first _ false].
	aStream nextPutAll: '))'! !


GesturalEvent subclass: #TalkGesturalEvent
	instanceVariableNames: 'phoneme '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Events'!

!TalkGesturalEvent methodsFor: 'accessing' stamp: 'len 9/6/1999 00:41'!
phoneme
	^ phoneme! !

!TalkGesturalEvent methodsFor: 'accessing' stamp: 'len 9/6/1999 00:41'!
phoneme: aPhoneme
	phoneme _ aPhoneme! !


!TalkGesturalEvent methodsFor: 'playing' stamp: 'len 9/6/1999 00:44'!
actOn: aHeadMorph
	aHeadMorph face lips articulate: self phoneme! !


!TalkGesturalEvent methodsFor: 'printing' stamp: 'len 9/7/1999 02:27'!
printOn: aStream
	aStream nextPutAll: 'articulate '; print: self phoneme! !

Voice initialize!