Object subclass: #Phoneme
	instanceVariableNames: 'name properties '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Phonetics'!
!Phoneme commentStamp: '<historical>' prior: 0!
My instances are phonemes. See also PhonemeSet class.!


!Phoneme methodsFor: 'accessing' stamp: 'len 6/9/1999 01:33'!
example
	^ self at: #example ifAbsent: []! !

!Phoneme methodsFor: 'accessing' stamp: 'len 6/9/1999 01:34'!
example: aString
	self at: #example put: aString! !

!Phoneme methodsFor: 'accessing' stamp: 'len 4/9/98 23:13'!
name
	^ name! !

!Phoneme methodsFor: 'accessing' stamp: 'len 6/9/1999 01:33'!
name: aString
	name _ aString! !

!Phoneme methodsFor: 'accessing' stamp: 'len 6/9/1999 01:35'!
stress
	"Answer the stress level of the receiver."
	^ self at: #stress ifAbsent: [0]! !

!Phoneme methodsFor: 'accessing' stamp: 'len 6/9/1999 01:38'!
stress: aNumber
	"Set the stress level of the receiver."
	self at: #stress put: aNumber! !


!Phoneme methodsFor: 'comparing' stamp: 'len 6/1/1999 02:13'!
= aPhoneme
	^ self species == aPhoneme species and: [self name = aPhoneme name]! !

!Phoneme methodsFor: 'comparing' stamp: 'len 5/29/1999 04:01'!
hash
	^ self name hash! !


!Phoneme methodsFor: 'copying' stamp: 'len 6/9/1999 02:10'!
copy
	^ super copy properties: properties copy! !


!Phoneme methodsFor: 'testing' stamp: 'len 6/9/1999 01:27'!
hasFeature: aSymbol
	^ self hasProperty: aSymbol! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isAffricate
	"Answer true if the receiver is an affricate phoneme."
	^ self hasFeature: #affricate! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isBackVowel
	"Answer true if the receiver is a back vowel phoneme."
	^ self isVowel and: [self hasFeature: #back]! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isConsonant
	"Answer true if the receiver is a consonant phoneme."
	^ self hasFeature: #consonant! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isContinuant
	"Answer true if the receiver is a continuant phoneme."
	^ self hasFeature: #continuant! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isDiphthong
	"Answer true if the receiver is a diphthong phoneme."
	^ self hasFeature: #diphthong! !

!Phoneme methodsFor: 'testing' stamp: 'len 8/10/1999 00:26'!
isFricative
	"Answer true if the receiver is a fricative phoneme."
	^ self hasFeature: #fricative! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isFrontVowel
	"Answer true if the receiver is a front vowel phoneme."
	^ self isVowel and: [self hasFeature: #front]! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:02'!
isGlide
	"Answer true if the receiver is a glide phoneme."
	^ self hasFeature: #glide! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isLiquid
	"Answer true if the receiver is a liquid phoneme."
	^ self hasFeature: #liquid! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isMidVowel
	"Answer true if the receiver is a mid vowel phoneme."
	^ self isVowel and: [self hasFeature: #mid]! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isNasal
	"Answer true if the receiver is an nasal phoneme."
	^ self hasFeature: #nasal! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 01:29'!
isNonContinuant
	"Answer true if the receiver is a noncontinuant phoneme."
	^ self isContinuant not! !

!Phoneme methodsFor: 'testing' stamp: 'len 8/16/1999 01:39'!
isObstruent
	"Answer true if the receiver is an obstruent phoneme."
	^ self isStop or: [self isFricative or: [self isAffricate]]! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isSemivowel
	"Answer true if the receiver is a semivowel phoneme."
	^ self hasFeature: #semivowel! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/8/1999 01:21'!
isSilence
	"Answer true if the receiver is the silence phoneme."
	^ self hasFeature: #silence! !

!Phoneme methodsFor: 'testing' stamp: 'len 8/16/1999 01:39'!
isSonorant
	"Answer true if the receiver is a sonorant phoneme."
	^ self isObstruent not! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isStop
	"Answer true if the receiver is a stop phoneme."
	^ self hasFeature: #stop! !

!Phoneme methodsFor: 'testing' stamp: 'len 8/16/1999 00:46'!
isSyllabic
	"Answer true if the receiver is a syllabic consonant (or a vowel)."
	^ self isVowel or: [self isDiphthong]! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isUnvoiced
	"Answer true if the receiver is an uvoiced phoneme."
	^ self hasFeature: #unvoiced! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isVoiced
	"Answer true if the receiver is a voiced phoneme."
	^ self hasFeature: #voiced! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/21/1999 19:55'!
isVoicedConsonant
	"Answer true if the receiver is a voiced consonant."
	^ self isVoiced and: [self isConsonant]! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isVowel
	"Answer true if the receiver is a vowel phoneme."
	^ self hasFeature: #vowel! !

!Phoneme methodsFor: 'testing' stamp: 'len 6/2/1999 02:03'!
isWhisper
	"Answer true if the receiver is an whisper phoneme."
	^ self hasFeature: #whisper! !


!Phoneme methodsFor: 'properties' stamp: 'len 6/9/1999 01:28'!
addProperty: anObject
	self at: anObject put: #nothing! !

!Phoneme methodsFor: 'properties' stamp: 'len 6/9/1999 01:26'!
at: anObject
	^ properties at: anObject! !

!Phoneme methodsFor: 'properties' stamp: 'len 6/9/1999 01:45'!
at: anObject ifAbsent: aBlock
	properties isNil ifFalse: [^ properties at: anObject ifAbsent: aBlock].
	^ aBlock value! !

!Phoneme methodsFor: 'properties' stamp: 'len 6/9/1999 01:27'!
at: anObject put: anotherObject
	properties isNil ifTrue: [properties _ IdentityDictionary new].
	^ properties at: anObject put: anotherObject! !

!Phoneme methodsFor: 'properties' stamp: 'len 6/10/1999 01:46'!
hasProperty: anObject
	^ properties notNil and: [properties includesKey: anObject]! !

!Phoneme methodsFor: 'properties' stamp: 'len 6/9/1999 01:30'!
properties
	^ properties! !

!Phoneme methodsFor: 'properties' stamp: 'len 6/9/1999 01:34'!
properties: anIdentityDictionary
	properties _ anIdentityDictionary! !


!Phoneme methodsFor: 'transforming' stamp: 'len 6/9/1999 01:35'!
stressed: aNumber
	^ self copy stress: aNumber! !


!Phoneme methodsFor: 'printing' stamp: 'len 8/17/1999 01:19'!
printOn: aStream
	name isNil ifTrue: [^ super printOn: aStream].
	aStream nextPutAll: name.
	self stress > 0 ifTrue: [aStream print: self stress]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Phoneme class
	instanceVariableNames: ''!

!Phoneme class methodsFor: 'instance creation' stamp: 'len 5/30/1999 01:17'!
name: aString
	^ self new name: aString! !

!Phoneme class methodsFor: 'instance creation' stamp: 'len 5/29/1999 04:34'!
name: aString example: anotherString
	^ self new name: aString; example: anotherString! !

!Phoneme class methodsFor: 'instance creation' stamp: 'len 6/9/1999 01:32'!
name: aString example: anotherString features: anArray
	| answer |
	answer _ self new name: aString; example: anotherString.
	anArray do: [ :each | answer addProperty: each].
	^ answer! !


Object subclass: #PhonemeSet
	instanceVariableNames: 'name description phonemes specials '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Phonetics'!
!PhonemeSet commentStamp: '<historical>' prior: 0!
My instances are phoneme sets, i.e. phonetic alphabets.
There are several "standard" phoneme sets used among phoneticians. We choosed ARPABET to be the default phoneme set in this system, but other examples are implemented, such as DARPA (radio), MRPA and SAMPA. As well, mappings from those phoneme sets to ARPABET are provided.!


!PhonemeSet methodsFor: 'initialization' stamp: 'len 6/2/1999 02:46'!
initialize
	phonemes _ Dictionary new.
	specials _ Dictionary new! !


!PhonemeSet methodsFor: 'accessing' stamp: 'len 5/31/1999 01:00'!
add: aPhoneme
	^ phonemes at: aPhoneme name put: aPhoneme! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 5/31/1999 03:45'!
at: aString
	^ phonemes at: aString! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 5/31/1999 03:45'!
at: aString ifAbsent: aBlock
	^ phonemes at: aString ifAbsent: aBlock! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/22/1998 00:53'!
description
	^ description! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/22/1998 00:53'!
description: aString
	description _ aString! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/22/1998 00:53'!
name
	^ name! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/22/1998 00:53'!
name: aString
	name _ aString! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/8/1999 00:55'!
names
	"Answer the names of all the phonemes."
	^ phonemes keys! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/5/1999 01:43'!
pause
	^ self silence! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 5/31/1999 01:00'!
phonemes
	^ phonemes values! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/5/1999 01:43'!
silence
	^ self specials at: #silence! !

!PhonemeSet methodsFor: 'accessing' stamp: 'len 6/2/1999 02:46'!
specials
	^ specials! !


!PhonemeSet methodsFor: 'copying' stamp: 'len 5/31/1999 01:35'!
copy
	^ self class new addAll: self phonemes; yourself! !


!PhonemeSet methodsFor: 'enumerating' stamp: 'len 8/22/1999 18:27'!
do: aBlock
	phonemes do: aBlock! !


!PhonemeSet methodsFor: 'transcribing' stamp: 'len 5/31/1999 03:26'!
transcriptionOf: aString
	^ (aString findTokens: '/ ')
		collect: [ :each |
			each last isDigit
				ifTrue: [(self at: (each copyFrom: 1 to: each size - 1))
							stressed: each last asString asNumber]
				ifFalse: [self at: each]]! !


!PhonemeSet methodsFor: 'printing' stamp: 'len 5/28/1999 02:55'!
printOn: aStream
	name isNil ifTrue: [^ super printOn: aStream].
	aStream nextPutAll: name, ' phoneme set'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PhonemeSet class
	instanceVariableNames: 'arpabet '!

!PhonemeSet class methodsFor: 'instance creation' stamp: 'len 6/22/1998 00:55'!
new
	^ super new initialize! !


!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/10/1999 04:12'!
arpabet
	"Answer the ARPAbet phoneme set."
	^ arpabet! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 12/24/1999 02:44'!
arpabetToSampa
	"Answer a dictionary with ARPAbet phonemes
	as keys and SAMPA phoneme names as values."

	| answer |
	answer _ Dictionary new.
	self sampaToArpabet associationsDo: [ :each | answer at: each value put: each key].
	^ answer! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/10/1999 04:12'!
darpa
	"Answer the DARPA phoneme set."
	^ self radio! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 12/20/1999 04:34'!
dectalkToArpabet
	"Answer a dictionary with DECTalk phoneme names
	as keys and ARPAbet phonemes as values."

	| answer |
	answer _ Dictionary new.
	self arpabet do: [ :each | answer at: each name put: each].
	#(
		('nx'	'ng')
		('yx'	'jh')
		('lx'		'l')
		('rr'	'r')
		('u'		'uw')
		('hx'	'hh')
		('h'		'hh')
		('_'		'sil')) do: [ :each | answer at: each first put: (self arpabet at: each last)].
	^ answer! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 5/31/1999 00:25'!
default
	^ self arpabet! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 9/29/1999 02:27'!
mactalkToArpabet
	"Answer a dictionary with MacTalk phoneme names
	as keys and ARPAbet phonemes as values."

	| answer |
	answer _ Dictionary new.
	#(	
		('IY'	'iy')
		('IH'	'ih')
		('EY'	'ey')
		('EH'	'eh')
		('AE'	'ae')
		('AA'	'aa')
		('AO'	'ao')
		('OW'	'ow')
		('UH'	'uh')
		('UW'	'uw')
"		(''	'er')"
		('AX'	'ax')
		('AH'	'ah')
		('AY'	'ay')
		('AW'	'aw')
		('OY'	'oy')
"		(''	'ix')"
		('p'		'p')
		('b'		'b')
		('t'		't')
		('d'		'd')
		('k'		'k')
		('g'		'g')
		('f'		'f')
		('v'		'v')
		('T'		'th')
		('D'		'dh')
		('s'		's')
		('z'		'z')
		('S'		'sh')
		('Z'		'zh')
		('h'		'hh')
		('m'		'm')
		('n'		'n')
		('N'		'ng')
		('l'		'l')
		('w'		'w')
		('y'		'y')
		('r'		'r')
		('C'		'ch')
		('J'		'jh')
		('UX'	'ax')
		('_'		'sil')
		('~'		'sil')) do: [ :each | answer at: each first put: (self arpabet at: each last)].
	^ answer! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/10/1999 04:11'!
mrpa
	"Answer the MRPA phoneme set."
	self notYetImplemented! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/21/1999 20:46'!
mrpaToArpabet
	"Answer a dictionary with MRPA phoneme names
	as keys and ARPAbet Phonemes as values."

	self notYetImplemented! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/8/1999 01:55'!
radio
	"Answer the radio phoneme set, named after the BU RADIO FM corpus."
	self notYetImplemented! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/10/1999 04:11'!
sampa
	"Answer the sampa phoneme set."
	| answer mapping |
	mapping _ self sampaToArpabet.
	answer _ self new.
	mapping keysDo: [ :each | answer add: ((self arpabet at: (mapping at: each)) copy name: each)].
	^ answer! !

!PhonemeSet class methodsFor: 'examples' stamp: 'len 6/12/1999 04:00'!
sampaToArpabet
	"Answer a dictionary with SAMPA phoneme names
	as keys and ARPAbet phonemes as values."

	| answer |
	answer _ Dictionary new.
	#(	('p'		'p')
		('b'		'b')
		('t'		't')
		('d'		'd')
		('k'		'k')
		('m'		'm')
		('n'		'n')
		('l'		'l')
		('r'		'r')
		('f'		'f')
		('v'		'v')
		('s'		's')
		('z'		'z')
		('h'		'hh')
		('w'		'w')
		('g'		'g')
		('tS'		'ch')
		('dZ'	'jh')
		('N'		'ng')
		('T'		'th')
		('D'		'dh')
		('S'		'sh')
		('Z'		'zh')
		('j'		'y')
		('i:'		'iy')
		('i'		'iy')
		('A:'	'aa')
		('A'		'aa')
		('O:'	'ao')
		('O'		'ao')
		('u:'	'uw')
		('u'		'uw')
		('3:'		'er')
		('r='	'er')
		('I'		'ih')
		('e'		'eh')
		('E'		'eh')
		('{'		'ae')
		('V'		'ah')
		"	('Q'		'oh')"
		('U'		'uh')
		('@'		'ax')
		('eI'		'ey')
		('aI'	'ay')
		('OI'	'oy')
		('@U'	'ow')
		('aU'	'aw')
		('I@'	'ia')
		('e@'	'ea')
		('U@'	'ua')
		('AI'	'ay')
		('EI'		'ey')
		('4'		't')
		('_'		'sil')) do: [ :each | answer at: each first put: (self arpabet at: each last)].
	^ answer! !


!PhonemeSet class methodsFor: 'class initialization' stamp: 'len 12/20/1999 04:30'!
initialize
	"
	PhonemeSet initialize
	"
	arpabet _ self new name: 'ARPAbet'; description: 'This is the ARPAbet phonetic alphabet.'.
	#("Name"	"Example"	"Features"
		('iy'	'heed'		#(continuant vowel front))
		('ih'	'hid'		#(continuant vowel front))
		('ey'	'hayed'		#(continuant vowel front))
		('eh'	'head'		#(continuant vowel front))
		('ae'	'had'		#(continuant vowel front))
		('aa'	'hod'		#(continuant vowel back))
		('ao'	'hawed'		#(continuant vowel back))
		('ow'	'hoed'		#(continuant vowel back))
		('uh'	'hood'		#(continuant vowel back))
		('uw'	'who''d'		#(continuant vowel back))
		('er'	'heard'		#(continuant vowel mid))
		('ax'	'ago'		#(continuant vowel mid))
		('ah'	'mud'		#(continuant vowel mid))
		('ay'	'hide'		#(diphthong))
		('aw'	'how''d'		#(diphthong))
		('oy'	'boy'		#(diphthong))
		('ix'		'roses'		#())
		('p'		'pea'		#(consonant stop unvoiced))
		('b'		'bat'		#(consonant stop voiced))
		('t'		'tea'		#(consonant stop unvoiced))
		('d'		'deep'		#(consonant stop voiced))
		('k'		'kick'		#(consonant stop unvoiced))
		('g'		'go'			#(consonant stop voiced))
		('f'		'five'		#(continuant consonant fricative unvoiced))
		('v'		'vice'		#(continuant consonant fricative voiced))
		('th'	'thing'		#(continuant consonant fricative unvoiced))
		('dh'	'then'		#(continuant consonant fricative voiced))
		('s'		'so'			#(continuant consonant fricative unvoiced))
		('z'		'zebra'		#(continuant consonant fricative voiced))
		('sh'	'show'		#(continuant consonant fricative unvoiced))
		('zh'	'measure'	#(continuant consonant fricative voiced))
		('hh'	'help'		#(continuant consonant whisper))
		('m'		'mom'		#(continuant consonant nasal))
		('n'		'noon'		#(continuant consonant nasal))
		('ng'	'sing'		#(continuant consonant nasal))		"old name: nx"
		('l'		'love'		#(semivowel liquid))
		('el'		'cattle')
		('em'	'some')
		('en'	'son')
		('dx'	'batter')
		('q'		'[glottal stop]')
		('w'		'want'		#(continuant semivowel glide))
		('y'		'yard'		#(continuant semivowel glide))
		('r'		'race'		#(continuant semivowel liquid))
		('ch'	'church'	#(continuant consonant affricate))
		('jh'	'just'		#(continuant consonant affricate))
		('wh'	'when'		#(semivowel liquid))
	"not found in the original:"
		('sil'	'[silence]'	#(silence))
		('ll'		'')	"dark l"
		('ai'	''			#(vowel))	"what is this?"
		('ia'	''			#(vowel))
		('ea'	''			#(vowel))
		('ua'	''			#(vowel))
		('aor'	'')
		('rx'	''			#())
	) do: [ :each |
		arpabet add: (each size > 2
						ifTrue: [Phoneme name: each first example: each second features: each last]
						ifFalse: [Phoneme name: each first example: each last])].
	arpabet specials at: #silence put: (arpabet at: 'sil')! !


Object subclass: #PhoneticRule
	instanceVariableNames: 'left text right phonemes '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Phonetics'!
!PhoneticRule commentStamp: '<historical>' prior: 0!
My instances are letter-to-sound rules for perform automatic text-to-phonemes transcription. (See the PhoneticTranscriber class comment too.)

Rules are made up of four parts: (1) left context pattern, (2) the text to match, (3) the right context pattern, and (4) the phonemes to substitute for the matched text.

The transcription procedure begins separating each block of letters (apostrophes included) and adding a space on each side. For each unmatched letter in the word, look through the rules where the text to match starts with the letter in the word. If the text to match if found and the right and left contexts patterns also match, output the phonemes for that rule and skip to the next unmatched letter.

Context patterns special characters:
	#	One or more vowels
	:	Zero or more consonants
	^	One consonant
	.	One of B, D, V, G, J, L, M, N, R, W, or Z (voiced consonants)
	%	One of ER, E, ES, ED, ING, ELY (a suffix)	[ONLY FOR RIGHT CONTEXT]
	+	One of E, I or Y (a "front" vowel)
Furthermore, the space character means any separator (space, comma, colon, etc), and any other character means that character it self.

The english example is derived from: "Automatic Translation of English Text to Phonetics by Means of Letter-To-Sound Rules", NRL Report 7948, January 21st, 1976, Naval Research Laboratory, Washington, D.C. Published by the National Technical Information Service as document "AD/A021 929".
!


!PhoneticRule methodsFor: 'accessing' stamp: 'len 5/29/1999 03:59'!
left
	^ left! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:46'!
left: aString
	left _ aString! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/10/98 05:01'!
phonemes
	^ phonemes! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:47'!
phonemes: aCollection
	phonemes _ aCollection! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 5/29/1999 03:59'!
right
	^ right! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:46'!
right: aString
	right _ aString! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/10/98 05:00'!
text
	^ text! !

!PhoneticRule methodsFor: 'accessing' stamp: 'len 4/9/98 23:46'!
text: aString
	text _ aString! !


!PhoneticRule methodsFor: 'testing' stamp: 'len 6/8/1999 00:14'!
matches: aString at: anInteger
	^ (self textMatches: aString at: anInteger)
		and: [(self leftMatches: aString at: anInteger)
			and: [self rightMatches: aString at: anInteger]]! !


!PhoneticRule methodsFor: 'private' stamp: 'len 11/3/1998 06:04'!
leftMatches: aString at: anInteger
	| leftindex textindex pattern |
	left isEmpty ifTrue: [^ true].
	leftindex _ left size.
	textindex _ anInteger - 1.
	[leftindex >= 1 and: [textindex >= 1]] whileTrue: [
		pattern _ left at: leftindex.
		"first check for simple text or apostrophe:"
		(pattern isAlphaNumeric or: [pattern = $'])
			ifTrue: [(aString at: textindex) asLowercase ~= pattern asLowercase
						ifTrue: [^ false].
					textindex _ textindex - 1].
		"space:"
		pattern = Character space
			ifTrue: [((aString at: textindex) isSeparator
						or: ['.,;:' includes: (aString at: textindex)]) ifFalse: [^ false].
					textindex _ textindex - 1].
		"one or more vowels:"
		pattern = $#
			ifTrue: [(aString at: textindex) isVowel ifFalse: [^ false].
					textindex _ textindex - 1.
					[textindex >= 1 and: [(aString at: textindex) isVowel]]
						whileTrue: [textindex _ textindex - 1]].
		"zero or more consonants:"
		pattern = $:
			ifTrue: [[textindex >= 1
						and: ['bcdfghjklmnpqrstvwxyz'
								includes: (aString at: textindex) asLowercase]]
							whileTrue: [textindex _ textindex - 1]].
		"one consonant:"
		pattern = $^
			ifTrue: [('bcdfghjklmnpqrstvwxyz' includes: (aString at: textindex))
						ifFalse: [^ false].
					textindex _ textindex - 1].
		"b, d, v, g, j, l, m, n, r, w, z (voiced consonants):"
		pattern = $.
			ifTrue: [('bdvgjlmnrwz' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex - 1].
		"e, i or y (front vowels)"
		pattern = $+
			ifTrue: [('eiy' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex - 1].
		leftindex _ leftindex - 1].
	^ true! !

!PhoneticRule methodsFor: 'private' stamp: 'len 11/3/1998 06:04'!
rightMatches: aString at: anInteger
	| rightindex textindex pattern |
	right isEmpty ifTrue: [^ true].
	rightindex _ 1.
	textindex _ anInteger + text size.
	[rightindex <= right size and: [textindex <= aString size]] whileTrue: [
		pattern _ right at: rightindex.
		"first check for simple text or apostrophe:"
		(pattern isAlphaNumeric or: [pattern = $'])
			ifTrue: [(aString at: textindex) asLowercase ~= pattern asLowercase
						ifTrue: [^ false].
					textindex _ textindex + 1].
		"space:"
		pattern = Character space
			ifTrue: [((aString at: textindex) isSeparator
						or: ['.,;:' includes: (aString at: textindex)]) ifFalse: [^ false].
					textindex _ textindex + 1].
		"one or more vowels:"
		pattern = $#
			ifTrue: [(aString at: textindex) isVowel ifFalse: [^ false].
					textindex _ textindex + 1.
					[textindex <= aString size and: [(aString at: textindex) isVowel]]
						whileTrue: [textindex _ textindex + 1]].
		"zero or more consonants:"
		pattern = $:
			ifTrue: [[textindex <= aString size
						and: ['bcdfghjklmnpqrstvwxyz'
								includes: (aString at: textindex) asLowercase]]
							whileTrue: [textindex _ textindex + 1]].
		"one consonant:"
		pattern = $^
			ifTrue: [('bcdfghjklmnpqrstvwxyz' includes: (aString at: textindex))
						ifFalse: [^ false].
					textindex _ textindex + 1].
		"b, d, v, g, j, l, m, n, r, w, z (voiced consonants):"
		pattern = $.
			ifTrue: [('bdvgjlmnrwz' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex + 1].
		"e, i or y (front vowels):"
		pattern = $+
			ifTrue: [('eiy' includes: (aString at: textindex) asLowercase)
						ifFalse: [^ false].
					textindex _ textindex + 1].
		"er, e, es, ed, ing, ely (a suffix):"
		pattern = $%
			ifTrue: [(aString at: textindex) asLowercase = $e
						ifTrue: [textindex _ textindex + 1.
								(textindex < aString size and: [(aString at: textindex) asLowercase = $l])
									ifTrue: [textindex _ textindex + 1.
											(textindex < aString size and: [(aString at: textindex) asLowercase = $y])
												ifTrue: [textindex _ textindex + 1]
												ifFalse: [textindex _ textindex - 1]]
									ifFalse: [('rsd' includes: (aString at: textindex) asLowercase)
												ifTrue: [textindex _ textindex + 1]]]
						ifFalse: [(textindex + 2 <= aString size
									and: [(aString at: textindex) asLowercase = $i
										and: [(aString at: textindex + 1) asLowercase = $n
											and: [(aString at: textindex + 2) asLowercase = $g]]])
									ifTrue: [textindex _ textindex + 3]
									ifFalse: [^ false]]].
		rightindex _ rightindex + 1].
	^ true! !

!PhoneticRule methodsFor: 'private' stamp: 'len 4/10/98 23:19'!
textMatches: aString at: anInteger
	text size > (aString size - anInteger + 1) ifTrue: [^ false].
	1 to: text size do: [ :each |
		(text at: each) asLowercase = (aString at: anInteger + each - 1) asLowercase ifFalse: [^ false]].
	^ true! !


!PhoneticRule methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:08'!
= anObject

	self species == anObject species ifFalse: [^ false].
	^ anObject left = self left
		and: [anObject right = self right
			and: [anObject text = self text
				and: [anObject phonemes = self phonemes]]]! !

!PhoneticRule methodsFor: 'comparing' stamp: 'len 5/29/1999 03:58'!
hash
	^ self text hash! !

!PhoneticRule methodsFor: 'comparing' stamp: 'tk 7/5/2001 22:09'!
species
	^PhoneticRule ! !


!PhoneticRule methodsFor: 'printing' stamp: 'len 5/31/1999 00:32'!
printOn: aStream
	aStream
		nextPut: $[; print: left; nextPut: $,; print: text; nextPut: $,; print: right; nextPut: $];
		nextPutAll: ' -> '.
	phonemes isEmpty ifTrue: [aStream nextPutAll: '{}'] ifFalse: [aStream nextPut: $/].
	phonemes do: [ :each | aStream nextPutAll: each name; nextPut: $/]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PhoneticRule class
	instanceVariableNames: ''!

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:14'!
english
	"Answer the english phonetic rules."
	| answer |
	answer _ OrderedCollection new.
	#(englishPunctuationRules
		englishARules englishBRules englishCRules englishDRules englishERules
		englishFRules englishGRules englishHRules englishIRules englishJRules
		englishKRules englishLRules englishMRules englishNRules englishORules
		englishPRules englishQRules englishRRules englishSRules englishTRules
		englishURules englishVRules englishWRules englishXRules englishYRules
		englishZRules) do: [ :each | answer addAll: (self perform: each)].
	^ answer asArray! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/30/1999 23:54'!
englishARules
	^ #((''		'a'		' '		'ax')
		(' '		'are'	' '		'aa/r')
		(' '		'ar'		'o'		'ax/r')
		(''		'ar'		'#'		'eh/r')
		('^'		'as'		'#'		'ey/s')
		(''		'a'		'wa'	'ax')
		(''		'aw'	''		'ao')
		(' :'		'any'	''		'eh/n/iy')
		(''		'a'		'^+#'	'ey')
		('#:'	'ally'	''		'ax/l/iy')
		(' '		'al'		'#'		'ax/l')
		(''		'again'	''		'ax/g/eh/n')
		('#:'	'ag'		'e'		'ih/jh')
		(''		'a'		'^+:#'	'ae')
		(' :'		'a'		'^+ '	'ey')
		(' '		'arr'	''		'ax/r')
		(''		'arr'	''		'ae/r')
		(' :'		'ar'		' '		'aa/r')
		(''		'ar'		' '		'er')
		(''		'ar'		''		'aa/r')
		(''		'air'	''		'eh/r')
		(''		'ai'		''		'ey')
		(''		'ay'		''		'ey')
		(''		'au'		''		'ao')
		('#:'	'al'		' '		'ax/l')
		('#:'	'als'	' '		'ax/l/z')
		(''		'alk'	''		'ao/k')
		(''		'al'		'^'		'ao/l')
		(' :'		'able'	''		'ey/b/ax/l')
		(''		'able'	''		'ax/b/ax/l')
		(''		'ang'	'+'		'ey/n/jh')
		('^'		'a'		'^#'		'ey')
		(''		'a'		''		'ae')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/30/1999 23:58'!
englishBRules
	^ #((' '		'be'		'^#'		'b/ih')
		(''		'being'	''		'b/iy/ih/ng')
		(' '		'both'	''		'b/ow/th')
		(' '		'bus'	'#'		'b/ih/z')
		(''		'buil'	''		'b/ih/l')
		(''		'b'		''		'b')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/30/1999 23:59'!
englishCRules
	^ #((' '		'ch'		'^'		'k')
		('^e'	'ch'		''		'k')
		(''		'ch'		''		'ch')
		(' s'		'ci'		'#'		's/ay')
		(''		'ci'		'a'		'sh')
		(''		'ci'		'o'		'sh')
		(''		'ci'		'en'		'sh')
		(''		'c'		'+'		's')
		(''		'ck'		''		'k')
		(''		'com'	'%'		'k/ah/m')
		(''		'c'		''		'k')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 12/2/1999 02:41'!
englishDRules
	^ #(('#:'	'ded'	' '		'd/ih/d')
		('.e'		'd'		' '		'd')
		('#:^e'	'd'		' '		't')
		(' '		'de'		'^#'		'd/ih')
		(' '		'do'		' '		'd/uw')
		(' '		'does'	''		'd/ah/z')
		(' '		'doing'	''		'd/uw/ih/ng')
		(' '		'dow'	''		'd/aw')
		(''		'du'		'a'		'jh/uw')
		(''		'd'		''		'd')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 12/9/1999 02:27'!
englishERules
	^ #(('#:'	'e'		' '		'')
		(''':^'	'e'		' '		'')
		(' :'		'e'		' '		'iy')
		('#'		'ed'		' '		'd')
		('#:'	'e'		'd'		'')
		(''		'ev'		'er'		'eh/v')
		(''		'e'		'^%'		'iy')
		(''		'eri'	'#'		'iy/r/iy')
		(''		'eri'	''		'eh/r/ih')
		('#:'	'er'		'#'		'er')
		(''		'er'		'#'		'eh/r')
		(''		'er'		''		'er')
		(' '		'even'	''		'iy/v/eh/n')
		('#:'	'e'		'w'		'')
		('t'		'ew'	''		'uw')
		('s'		'ew'	''		'uw')
		('r'		'ew'	''		'uw')
		('d'		'ew'	''		'uw')
		('l'		'ew'	''		'uw')
		('z'		'ew'	''		'uw')
		('n'		'ew'	''		'uw')
		('j'		'ew'	''		'uw')
		('th'	'ew'	''		'uw')
		('ch'	'ew'	''		'uw')
		('sh'	'ew'	''		'uw')
		(''		'ew'	''		'y/uw')
		(''		'e'		'o'		'iy')
		('#:s'	'es'		' '		'ih/z')
		('#:c'	'es'		' '		'ih/z')
		('#:g'	'es'		' '		'ih/z')
		('#:z'	'es'		' '		'ih/z')
		('#:x'	'es'		' '		'ih/z')
		('#:j'	'es'		' '		'ih/z')
		('#:ch'	'es'		' '		'ih/z')
		('#:sh'	'es'		' '		'ih/z')
		('#:'	'e'		's'		'')
		('#:'	'ely'	' '		'l/iy')
		('#:'	'ement'	''		'm/eh/n/t')
		(''		'eful'	''		'f/uh/l')
		(''		'ee'		''		'iy')
		(''		'earn'	''		'er/n')
		(' '		'ear'	'^'		'er')
		(''		'ead'	''		'eh/d')
		('#:'	'ea'		' '		'iy/ax')
		(''		'ea'		'su'		'eh')
		(''		'ea'		''		'iy')
		(''		'eigh'	''		'ey')
		(''		'ei'		''		'iy')
		(' '		'eye'	''		'ay')
		(''		'ey'		''		'iy')
		(''		'eu'		''		'y/uw')
		(''		'e'		''		'eh')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:01'!
englishFRules
	^ #((''		'ful'	''		'f/uh/l')
		(''		'f'		''		'f')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:02'!
englishGRules
	^ #((''		'giv'	''		'g/iy/v')
		(' '		'g'		'i^'		'g')
		(''		'ge'		't'		'g/eh')
		('su'	'gges'	''		'g/jh/eh/s')
		(''		'gg'		''		'g')
		(' b#'	'g'		''		'g')
		(''		'g'		'+'		'jh')
		(''		'great'	''		'g/r/ey/t')
		('#'		'gh'		''		'')
		(''		'g'		''		'g')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 7/20/1999 00:42'!
englishHRules
	^ #((' '		'hav'	''		'hh/ae/v')
		(' '		'here'	''		'hh/iy/r')
		(' '		'hour'	''		'aw/er')
		(''		'how'	''		'hh/aw')
		(''		'h'		'#'		'hh')
		(''		'h'		''		'')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:03'!
englishIRules
	^ #((' '		'in'		''		'ih/n')
		(' '		'i'		' '		'ay')
		(''		'in'		'd'		'ay/n')
		(''		'ier'	''		'iy/er')
		('#:r'	'ied'	''		'iy/d')
		(''		'ied'	' '		'ay/d')
		(''		'ien'	''		'iy/eh/n')
		(''		'ie'		't'		'ay/eh')
		(' :'		'i'		'%'		'ay')
		(''		'i'		'%'		'iy')
		(''		'ie'		''		'iy')
		(''		'i'		'^+:#'	'ih')
		(''		'ir'		'#'		'ay/r')
		(''		'iz'		'%'		'ay/z')
		(''		'is'		'%'		'ay/z')
		(''		'i'		'd%'		'ay')
		('+^'	'i'		'^+'		'ih')
		(''		'i'		't%'		'ay')
		('#:^'	'i'		'^+'		'ih')
		(''		'i'		'^+'		'ay')
		(''		'ir'		''		'er')
		(''		'igh'	''		'ay')
		(''		'ild'		''		'ay/l/d')
		(''		'ign'	' '		'ay/n')
		(''		'ign'	'^'		'ay/n')
		(''		'ign'	'%'		'ay/n')
		(''		'ique'	''		'iy/k')
		(''		'i'		''		'ih')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:03'!
englishJRules
	^ #((''		'j'		''		'jh')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:04'!
englishKRules
	^ #((''		'k'		'n'		'')
		(''		'k'		''		'k')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:04'!
englishLRules
	^ #((''		'lo'		'c#'		'l/ow')
		('l'		'l'		''		'')
		('#:^'	'l'		'%'		'ax/l')
		(''		'lead'	''		'l/iy/d')
		(''		'l'		''		'l')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:04'!
englishMRules
	^ #((''		'mov'	''		'm/uw/v')
		(''		'm'		''		'm')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:05'!
englishNRules
	^ #(('e'		'ng'		'+'		'n/jh')
		(''		'ng'		'r'		'ng/g')
		(''		'ng'		'#'		'ng/g')
		(''		'ngl'	'%'		'ng/g/ax/l')
		(''		'ng'		''		'ng')
		(''		'nk'		''		'ng/k')
		(' '		'now'	' '		'n/aw')
		(''		'n'		''		'n')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:06'!
englishORules
	^ #((''		'of'		' '		'ax/v')
		(''		'orough'	''	'er/ow')
		('#:'	'or'		' '		'er')
		('#:'	'ors'	' '		'er/z')
		(''		'or'		''		'ao/r')
		(' '		'one'	''		'w/ah/n')
		(''		'ow'	''		'ow')
		(' '		'over'	''		'ow/v/er')
		(''		'ov'		''		'ah/v')
		(''		'o'		'^%'		'ow')
		(''		'o'		'^en'	'ow')
		(''		'o'		'^i#'	'ow')
		(''		'ol'		'd'		'ow/l')
		(''		'ought'	''		'ao/t')
		(''		'ough'	''		'ah/f')
		(' '		'ou'		''		'aw')
		('h'		'ou'		's#'		'aw')
		(''		'ous'	''		'ax/s')
		(''		'our'	''		'ao/r')
		(''		'ould'	''		'uh/d')
		('^'		'ou'		'^l'		'ah')
		(''		'oup'	''		'uw/p')
		(''		'ou'		''		'aw')
		(''		'oy'		''		'oy')
		(''		'oing'	''		'ow/ih/ng')
		(''		'oi'		''		'oy')
		(''		'oor'	''		'ao/r')
		(''		'ook'	''		'uh/k')
		(''		'ood'	''		'uh/d')
		(''		'oo'		''		'uw')
		(''		'o'		'e'		'ow')
		(''		'o'		' '		'ow')
		(''		'oa'		''		'ow')
		(' '		'only'	''		'ow/n/l/iy')
		(' '		'once'	''		'w/ah/n/s')
		(''		'on''t'	''		'ow/n/t')
		('c'		'o'		'n'		'aa')
		(''		'o'		'ng'		'ao')
		(' :^'	'o'		'n'		'ah')
		('i'		'on'		''		'ax/n')
		('#:'	'on'		' '		'ax/n')
		('#^'	'on'		''		'ax/n')
		(''		'o'		'st '		'ow')
		(''		'of'		'^'		'ao/f')
		(''		'other'	''		'ah/dh/er')
		(''		'oss'	' '		'ao/s')
		('#:^'	'om'	''		'ah/m')
		(''		'o'		''		'aa')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:06'!
englishPRules
	^ #((''		'ph'		''		'f')
		(''		'peop'	''		'p/iy/p')
		(''		'pow'	''		'p/aw')
		(''		'put'	' '		'p/uh/t')
		(''		'p'		''		'p')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:07'!
englishPunctuationRules
	^ #(('.'		'''s'		''		'z')
		('#:.e'	'''s'		''		'z')
		('#'		'''s'		''		'z')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:07'!
englishQRules
	^ #((''		'quar'	''		'k/w/ao/r')
		(''		'qu'		''		'k/w')
		(''		'q'		''		'k')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:07'!
englishRRules
	^ #((''		're'		'^#'		'r/iy')
		(''		'r'		''		'r')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:07'!
englishSRules
	^ #((''		'sh'		''		'sh')
		('#'		'sion'	''		'zh/ax/n')
		(''		'some'	''		's/ah/m')
		('#'		'sur'	'#'		'zh/er')
		(''		'sur'	'#'		'sh/er')
		('#'		'su'		'#'		'zh/uw')
		('#'		'ssu'	'#'		'sh/uw')
		('#'		'sed'	' '		'z/d')
		('#'		's'		'#'		'z')
		(''		'said'	''		's/eh/d')
		('^'		'sion'	''		'sh/ax/n')
		(''		's'		's'		'')
		('.'		's'		' '		'z')
		('#:.e'	's'		' '		'z')
		('#:^##'	's'		' '		'z')
		('#:^#'	's'		' '		's')
		('u'		's'		' '		's')
		(' :#'	's'		' '		'z')
		(' '		'sch'	''		's/k')
		(''		's'		'c+'		'')
		('#'		'sm'		''		'z/m')
		('#'		'sn'		''''		'z/ax/n')
		(''		's'		''		's')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:08'!
englishTRules
	^ #((' '		'the'	' '		'dh/ax')
		(''		'to'		' '		't/uw')
		(''		'that'	' '		'dh/ae/t')
		(' '		'this'	' '		'dh/ih/s')
		(' '		'they'	''		'dh/ey')
		(' '		'there'	''		'dh/eh/r')
		(''		'ther'	''		'dh/er')
		(''		'their'	''		'dh/eh/r')
		(' '		'than'	' '		'dh/ae/n')
		(' '		'them'	' '		'dh/eh/m')
		(''		'these'	' '		'dh/iy/z')
		(' '		'then'	''		'dh/eh/n')
		(''		'through'	''	'th/r/uw')
		(''		'those'	''		'dh/ow/z')
		(''		'though'	' '	'dh/ow')
		(' '		'thus'	''		'dh/ah/s')
		(''		'th'		''		'th')
		('#:'	'ted'	' '		't/ih/d')
		('s'		'ti'		'#n'	'ch')
		(''		'ti'		'o'		'sh')
		(''		'ti'		'a'		'sh')
		(''		'tien'	''		'sh/ax/n')
		(''		'tur'	'#'		'ch/er')
		(''		'tu'		'a'		'ch/uw')
		(''		'two'	''		't/uw')
		(''		't'		''		't')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:09'!
englishURules
	^ #((' '		'un'	'i'		'y/uw/n')
		(' '		'un'	''		'ah/n')
		(' '		'upon'	''		'ax/p/ao/n')
		('t'		'ur'		'#'		'uh/r')
		('s'		'ur'		'#'		'uh/r')
		('r'		'ur'		'#'		'uh/r')
		('d'		'ur'		'#'		'uh/r')
		('l'		'ur'		'#'		'uh/r')
		('z'		'ur'		'#'		'uh/r')
		('n'		'ur'		'#'		'uh/r')
		('j'		'ur'		'#'		'uh/r')
		('th'	'ur'		'#'		'uh/r')
		('ch'	'ur'		'#'		'uh/r')
		('sh'	'ur'		'#'		'uh/r')
		(''		'ur'		'#'		'y/uh/r')
		(''		'ur'		''		'er')
		(''		'u'		'^'		'ah')
		(''		'u'		'^^'		'ah')
		(''		'uy'	''		'ay')
		(' g'	'u'		'#'		'')
		('g'		'u'		'%'		'')
		('g'		'u'		'#'		'w')
		('#n'	'u'		''		'y/uw')
		('t'		'u'		''		'uw')
		('s'		'u'		''		'uw')
		('r'		'u'		''		'uw')
		('d'		'u'		''		'uw')
		('l'		'u'		''		'uw')
		('z'		'u'		''		'uw')
		('n'		'u'		''		'uw')
		('j'		'u'		''		'uw')
		('th'	'u'		''		'uw')
		('ch'	'u'		''		'uw')
		('sh'	'u'		''		'uw')
		(''		'u'		''		'y/uw')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:09'!
englishVRules
	^ #((''		'view'	''		'v/y/uw')
		(''		'v'		''		'v')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 7/20/1999 00:43'!
englishWRules
	^ #((' '		'were'	''		'w/er')
		(''		'wa'	's'		'w/aa')
		(''		'wa'	't'		'w/aa')
		(''		'where'	''		'wh/eh/r')
		(''		'what'	''		'wh/aa/t')
		(''		'whol'	''		'hh/ow/l')
		(''		'who'	''		'hh/uw')
		(''		'wh'	''		'wh')
		(''		'war'	''		'w/ao/r')
		(''		'wor'	'^'		'w/er')
		(''		'wr'	''		'r')
		(''		'w'		''		'w')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:10'!
englishXRules
	^ #((''		'x'		''		'k/s')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:10'!
englishYRules
	^ #((''		'young'	''		'y/ah/ng')
		(' '		'you'	''		'y/uw')
		(' '		'yes'	''		'y/eh/s')
		(' '		'y'		''		'y')
		('#:^'	'y'		' '		'iy')
		('#:^'	'y'		'i'		'iy')
		(' :'		'y'		''		'ay')
		(' :'		'y'		'#'		'ay')
		(' :'		'y'		'^+:#'	'ih')
		(' :'		'y'		'^#'		'ay')
		(''		'y'		''		'ih')
	) collect: [ :each | self fromArray: each]! !

!PhoneticRule class methodsFor: 'examples-english' stamp: 'len 5/31/1999 00:10'!
englishZRules 
	^ #((''		'z'		''		'z')
	) collect: [ :each | self fromArray: each]! !


!PhoneticRule class methodsFor: 'examples' stamp: 'len 12/8/1999 17:14'!
spanish
	"Answer the spanish phonetic rules."
	| phonemes |
	phonemes _ PhonemeSet spanish.
	^ #(
		(''		'a'		''		'a')
		(''		'b'		''		'b')
		(''		'ch'		''		'ch')
		(''		'c'		'i'		'z')
		(''		'c'		'e'		'z')
		(''		'c'		''		'k')
		(''		'd'		''		'd')
		(''		'e'		''		'e')
		(''		'f'		''		'f')
		(''		'g'		'e'		'j')
		(''		'g'		'i'		'j')
		(''		'gu'		'e'		'g')
		(''		'gu'		'i'		'g')
		(''		'g'		''		'g')
		(''		'h'		''		'')
		(''		'i'		''		'i')
		(''		'j'		''		'j')
		(''		'k'		''		'k')
		(''		'll'		''		'y')
		(''		'l'		''		'l')
		(''		'm'		''		'm')
		(''		'n'		''		'n')
		(''		'o'		''		'o')
		(''		'p'		''		'p')
		(''		'qu'		''		'k')
		(''		'rr'		''		'rx')
		(' '		'r'		''		'rx')
		(''		'r'		''		'r')
		(''		's'		''		's')
		(''		't'		''		't')
		(''		'u'		''		'u')
		(''		'v'		''		'v')
		(''		'w'		''		'w')
		(''		'x'		''		'k/s')
		(''		'y'		' '		'i')
		(''		'y'		''		'y')
		(''		'z'		''		'z')
	) collect: [ :each | self fromArray: each phonemes: phonemes]! !


!PhoneticRule class methodsFor: 'instance creation' stamp: 'len 5/31/1999 00:16'!
fromArray: anArray
	^ self fromArray: anArray phonemes: PhonemeSet arpabet! !

!PhoneticRule class methodsFor: 'instance creation' stamp: 'len 5/31/1999 00:19'!
fromArray: anArray phonemes: aPhonemeSet
	^ self new
		left: (anArray at: 1);
		text: (anArray at: 2);
		right: (anArray at: 3);
		phonemes: (aPhonemeSet transcriptionOf: (anArray at: 4)) asArray! !


Object subclass: #PhoneticTranscriber
	instanceVariableNames: 'phonemes rules lexicon '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Phonetics'!
!PhoneticTranscriber commentStamp: '<historical>' prior: 0!
My instances perform automatic words-to-phonemes transcription. (See the PhoneticRule class comment too.)

Each transcriber must have a collection of PhoneticRules, and optionally a lexicon. When a transcriber is asked for the transcription of a word, it searches for the word in the lexicon first, and if the word is not found then the rules are used.

Rules are made up of four parts: (1) left context pattern, (2) the text to match, (3) the right context pattern, and (4) the phonemes to substitute for the matched text.

The transcription procedure begins when a text is provided to a transcriber. For each unmatched letter in the word, look through the rules where the text to match starts with the letter in the word. If one matching rule is found, then the rule is applied, writing the corresponding phonemes in the output and moving forward to the next unmatched position. If no rule is found for a position in the text, then the unmatched position is logged. At the end of the transcription, the phonemes extracted are provided in a collection and so are the unmatched positions.
!


!PhoneticTranscriber methodsFor: 'accessing-private' stamp: 'len 11/29/1999 02:31'!
lexicon: aDictionary
	lexicon _ aDictionary! !

!PhoneticTranscriber methodsFor: 'accessing-private' stamp: 'len 6/5/1999 01:56'!
phonemes: aPhonemeSet
	phonemes _ aPhonemeSet! !

!PhoneticTranscriber methodsFor: 'accessing-private' stamp: 'len 4/10/98 05:10'!
rules: aCollection
	rules _ aCollection! !


!PhoneticTranscriber methodsFor: 'accessing' stamp: 'len 11/29/1999 02:31'!
lexicon
	^ lexicon! !

!PhoneticTranscriber methodsFor: 'accessing' stamp: 'len 6/5/1999 01:56'!
phonemes
	^ phonemes! !

!PhoneticTranscriber methodsFor: 'accessing' stamp: 'len 6/5/1999 01:56'!
rules
	^ rules! !


!PhoneticTranscriber methodsFor: 'computing' stamp: 'len 12/14/1999 03:39'!
transcriptionOf: aString
	"Answer the phonetic transcription of the word in aString."
	| rule string index transcription stressed |
	(transcription _ self tryLexicon: aString) isNil ifFalse: [^ transcription].
	transcription _ OrderedCollection new.
	string _ ' ', aString,' '.
	index _ 2.
	[index < string size] whileTrue: [
		rule _ self rules
			detect: [ :one | one matches: string at: index]
			ifNone: [].
		rule isNil
			ifTrue: ["unmatched character" index _ index+1]
			ifFalse: [index _ index + rule text size.
					transcription addAll: rule phonemes]].
	stressed _ false.
	^ transcription collect: [ :each |
		(stressed not and: [each isVowel or: [each isDiphthong]])
			ifTrue: [stressed _ true. each stressed: 1] ifFalse: [each]]! !


!PhoneticTranscriber methodsFor: 'computing-private' stamp: 'len 11/29/1999 02:31'!
tryLexicon: aWord
	| string |
	self lexicon isNil ifTrue: [^ nil].
	string _ self lexicon at: aWord asUppercase ifAbsent: [^ nil].
	^ self phonemes transcriptionOf: string asLowercase! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PhoneticTranscriber class
	instanceVariableNames: ''!

!PhoneticTranscriber class methodsFor: 'examples' stamp: 'len 4/13/98 16:21'!
default
	^ self english! !

!PhoneticTranscriber class methodsFor: 'examples' stamp: 'len 12/14/1999 03:29'!
english
	"Answer an english phonetic transcriber."
	^ self new rules: PhoneticRule english; phonemes: PhonemeSet arpabet; lexicon: self englishLexicon! !

!PhoneticTranscriber class methodsFor: 'examples' stamp: 'len 8/24/2001 13:48'!
englishLexicon
	^ Dictionary new
		add: 'HOW' -> 'HH AW1';
		add: 'YOU' -> 'Y UW1';
		add: 'ARE' -> 'AA1 R';
		add: 'DOING' -> 'D UW1 IH0 NG';
		add: 'THIS' -> 'DH IH1 S';
		add: 'IS' -> 'IH1 Z';
		add: 'MY' -> 'M AY1';
		add: 'HI' -> 'HH AY1';
		add: 'VOICE' -> 'V OY1 S';
		add: 'FAST' -> 'F AE1 S T';
		add: 'SLOW' -> 'S L OW1';
		add: 'I' -> 'AY1';
		add: 'AM' -> 'AE1 M';
		add: 'A' -> 'AH0';
		add: 'AN' ->  'AE1 N';
		add: 'LOW' -> 'L OW1';
		add: 'SPEAKER' -> 'S P IY1 K ER0';
		add: 'ANSWER' -> 'AE1 N S ER0';
		add: 'RECEIVER' -> 'R AH0 S IY1 V ER0';
		add: 'OBJECT' -> 'AA1 B JH EH0 K T';
		add: 'READ' -> 'R IY1 D';
		add: 'WRITE' -> 'R AY1 T';
		add: 'SQUEAK' -> 'S K W IY1 K';
		add: 'SMALLTALK' -> ' S M AO1 L T AO2 K';
		add: 'CLASS' -> 'K L AE1 S';
		add: 'WOMAN' -> 'W UH1 M AH0 N';
		add: 'BICYCLIC' ->  'B AY1 S IH0 K L IH0 K';
		add: 'LISTEN' -> 'L IH1 S AH0 N';
		add: 'ZERO' -> 'Z IY1 R OW';
		add: 'SEVEN' -> 'S EH1 V EH N';
		add: 'ELEVEN' -> 'EH1 L EH1 V EH N';
		add: 'SEVENTEEN' -> 'S EH1 V EH N T IY N';
		add: 'SEVENTY' -> 'S EH1 V EH N T IH';
		add: 'NINETEEN' -> 'N AH1 N T IY N';
		add: 'NINETY' -> 'N AH1 N T IH';
		yourself! !

!PhoneticTranscriber class methodsFor: 'examples' stamp: 'len 12/14/1999 03:29'!
spanish
	"Answer a spanish phonetic transcriber."
	^ self new rules: PhoneticRule spanish; phonemes: PhonemeSet arpabet! !

PhonemeSet initialize!